function [ indexset_cell, coeff_cell ] = build_signals_1d( S, M, type, num_signals, filename )
%BUILD_SIGNALS_1D Creates sparse random one-dimensional test signals.
%
%   Input parameters:
%      S - parameter for index set (-S/2,...,S/2] of possible frequencies
%      M - signal sparsity
%      type - type of Fourier coefficients,
%             'unit_circle' for uniform random on the complex unit circle,
%             'box_gr1e-2' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-2
%             'box_gr1e-1' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-1
%      num_signals - number of test signals to generate
%      filename - output file name for test signals
%
%   Output parameters:
%      indexset_cell - cell array (length num_signals) containing column vectors
%                      of length M of frequencies
%      coeff_cell - cell array (length num_signals) containing column vectors
%                   of length M of Fourier coefficients
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if exist(filename,'file') ~= 0
  error('File %s already exists', filename);
end

indexset_cell = cell(num_signals,1);
coeff_cell = cell(num_signals,1);

for ir=1:num_signals
  indexset_in = [];
  while length(indexset_in) < M
    indexset_in = unique([indexset_in; round(S*rand(2*M,1)+0.5) - S/2], 'stable');
  end
  if length(indexset_in) > M
    indexset_in = indexset_in(1:M);
  end
  indexset_cell{ir} = indexset_in;

  switch type
    case 'unit_circle'
      fc_min = 1;
      coeff_in = exp(2*pi*1i*rand(M,1));
    case 'box_gr1e-2'
      fc_min = 1e-2;
      coeff_in = [];
      while length(coeff_in) < M
        coeff_in_new = 2*rand(M,1)-1 + 2i*rand(M,1)-1i;
        coeff_in_new = coeff_in_new(abs(coeff_in_new) >= 1e-2);
        coeff_in = [coeff_in; coeff_in_new];
      end
      coeff_in = coeff_in(1:M);
    case 'box_gr1e-1'
      fc_min = 1e-1;
      coeff_in = [];
      while length(coeff_in) < M
        coeff_in_new = 2*rand(M,1)-1 + 2i*rand(M,1)-1i;
        coeff_in_new = coeff_in_new(abs(coeff_in_new) >= 1e-1);
        coeff_in = [coeff_in; coeff_in_new];
      end
      coeff_in = coeff_in(1:M);
    otherwise
      error('type %s not supported', type);
  end
  coeff_cell{ir} = coeff_in;
end

save(filename, 'indexset_cell', 'coeff_cell', 'num_signals', 'S', 'fc_min');

end
