function [ M, Omega_tilde, fourier_coeff ] = prony_esprit_svd( vector_h, L, N, cut_off_epsilon )
%PRONY_ESPRIT_SVD ESPRIT method based on SVD of Hankel matrix.
%This algorithm detects the sparsity, (fractional) frequencies and
%coefficients of a time series of length N.
%For further details, please read
%
%   D. Potts, M. Tasche, T. Volkmer
%   Efficient spectral estimation by MUSIC and ESPRIT with application to sparse FFT.
%   Front. Appl. Math. Stat., accepted, 2016.
%
%This is Algorithm 2.9 from the paper.
%
%   Input parameters:
%      vector_h - samples of an exponential sum as a vector of length N,
%                 (time series of length N)
%      L - window length
%      N - number of samples, created Hankel matrix has size [L,(N-L+1)]
%      cut_off_epsilon - tolerance / cut-off parameter for detection of sparsity
%
%   Output parameters:
%      M - detected sparsity of signal
%      Omega_tilde - detected (fractional) frequencies
%      fourier_coeff - corresponding coefficients
%
% Copyright (c) 2015-2016 Daniel Potts, Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if length(vector_h) ~= N
  error('input vector_h must be of length N=%d but is of length %d', N, length(vector_h));
end

if (cut_off_epsilon < 0) || (cut_off_epsilon > 1)
  error('input cut_off_epsilon must be in the interval [0,1]');
end

% step 1
% create Hankel matrix from times series
matrix_H = hankel(vector_h(1:L), vector_h(L:end));
%[L, N_minus_L_plus_1] = size(matrix_H);

% perform SVD on Hankel matrix
[~,D,W] = svd(matrix_H);
diag_D = diag(D);

if isnan(cut_off_epsilon)
  % if cut-off parameter epsilon is not given, perform cut-off by largest
  % quotient of consecutive singular values
  rel = diag_D(1:end-1)./diag_D(2:end);
  [~,ind] = max(rel);
  diag_D_nonzero = diag_D(1:ind);
else
  % determine singular values >= cut_off_epsilon * largest_singular_value
  diag_D_nonzero = diag_D(diag_D >= cut_off_epsilon*diag_D(1));
end

% detected sparsity
M = length(diag_D_nonzero);

if isnan(cut_off_epsilon) && (M>=L-1)
  M = L;
end

if L > N - M + 1
  error('condition L <= N-M+1 is not fulfilled');
end

W1 = W(1:end-1,1:M);
W2 = W(2:end,1:M);

% step 2 and 3
phi=eig(W1'/W2');
Omega_tilde = -(imag(log(phi)))/2/pi;

% step 4
if nargout >= 3
  A_sparse = exp(2*pi*1i* (0:N-1)' * Omega_tilde');
  rank_A_sparse = rank(A_sparse);
  if rank_A_sparse ~= M
    error('rank of Vandermonde matrix = %d < detected sparsity M = %d', rank_A_sparse, M);
  end

  % compute coefficients as least-squares solution
  fourier_coeff = A_sparse \ vector_h;
end

end
