function [ indexset_I, fourier_coeff, niter_used, nsamples, p_vector, fourier_coeff_full ] = prony_sparsefft_iterative_algorithm( fct_handle, algorithm_name, S, initial_P, K, K_SVD_threshold, epsilon_svd_list, epsilon_spatial, threshold_fc, niter_max, show_debugmsg, use_l1, tol_l1 )
%PRONY_SPARSEFFT_ITERATIVE_ALGORITHM Sparse FFT via MUSIC/ESPRIT based on
%shifted sampling.
%This is an iterative algorithm for the reconstruction of sparse
%trigonometric polynomials which determines the sparsity, the index set of
%frequencies belonging to the non-zero Fourier coefficients and the Fourier
%coefficients from samples. For further details, please read
%
%   D. Potts, M. Tasche, T. Volkmer
%   Efficient spectral estimation by MUSIC and ESPRIT with application to sparse FFT.
%   Front. Appl. Math. Stat., accepted, 2016.
%
%This is Algorithm 6.1 from the paper.
%
%   Input parameters:
%      fct_handle - function handle (block box) which accepts sampling
%                   nodes as an array of size [num_nodes,d]
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      S - parameter for index set (-S/2,...,S/2] of possible frequencies
%      initial_P - initial FFT length P for first iteration
%      K - size of Hankel matrix used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold - cut-off parameter for number of detected frequencies,
%                        which determines if results of local MUSIC/ESPRIT run
%                        are used, e.g. choose equal to input parameter K
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%      epsilon_spatial - spatial cut-off parameter
%      threshold_fc - magnitude of smallest possible Fourier coefficient
%      niter_max - maximal number of allowed iterations
%      show_debugmsg - (optional) parameter for displaying debug messages,
%                      either 1 for enabled
%                      or 0 (default) for disabled
%      use_l1 - (optional) parameter which determines if l1 or l2 solver is
%               used for determining the Fourier coefficients,
%               either 1 for SPGL1 solver (by van den Berg and Friedlander)
%               or 0 (default) for MATLAB least-squares solver (backslash op.)
%      tol_l1 - tolerance parameter for SPGL1 solver (if enabled)
%
%   Output parameters:
%      indexset_I - column vector of detected frequencies,
%                   detected sparsity is length of this vector
%      fourier_coeff - column vector of corresponding Fourier coefficients
%                      obtained by solving small local Vandermonde systems
%      niter_used - number of used iterations
%      nsamples - total number of used samples
%      p_vector - rows vector of FFT lengths of length niter_max
%      fourier_coeff_full - column vector of Fourier coefficients computed
%                           by solving one large Vandermonde system which
%                           uses all samples and all detected frequencies
%
% Copyright (c) 2015-2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if nargin < 12
  use_l1 = 0;
end

if use_l1
  fprintf('l1 solver, ');
else
  fprintf('l2 solver, ');
end

switch algorithm_name
  case 'music'
    fprintf('iterative sparse FFT algorithm via MUSIC\n');
  case 'esprit'
    fprintf('iterative sparse FFT algorithm via ESPRIT\n');
  otherwise
    error('input algorithm must be either music or esprit but is %s\n', algorithm_name);
end

if (mod(S,2) ~= 0) || (S < 2)
  error('input S must be even positive integer');
end

if initial_P < 1
  error('input initial_p must be at least 1');
end

if K < 1
  error('input K must be at least 1');
end

if (K==1) && (initial_P~=S)
  error('if K==1 then initial_p must be equal to S');
end

if (K_SVD_threshold < 1) || (K_SVD_threshold > K)
  error('input K_SVD_threshold must be from [1,...,K]');
end

if (min(epsilon_svd_list) < 0) || (max(epsilon_svd_list) > 1)
  error('values of input epsilon_svd_list must be in the interval [0,1]');
end

if epsilon_spatial < 0
  error('input epsilon_spatial must be non-negative');
end

if threshold_fc < 0
  error('input epsilon_spatial must be non-negative');
end


if niter_max < 1
  error('input niter_max must be at least 1');
end

if use_l1
  opts = spgSetParms('verbosity',0,'iterations',1000,'optTol',tol_l1,'bpTol',tol_l1);
end

indexset_I = [];
fourier_coeff = [];
nsamples = 0;

if nargin < 9
  show_debugmsg = 0;
end

if niter_max >= 2
  p_vector = [initial_P, find_nextprimes(initial_P, niter_max-1)];
else
  p_vector = initial_P;
end

vector_signal_all = [];
vector_x_all = [];

if K==1
  freq_grid = [(0:S/2)'; (-S/2+1:-1)'];
  x = (0:S-1)'/S;
  fct_vals = fct_handle(x);
  fourier_coeff = fft(fct_vals) / S;
  ind = find(abs(fourier_coeff) >= threshold_fc);
  indexset_I = freq_grid(ind);
  fourier_coeff = fourier_coeff(ind);
  fourier_coeff_full = fourier_coeff;
  niter_used = 1;
else
  for niter=1:niter_max
    p = p_vector(niter);
    if show_debugmsg
      fprintf('start iteration %d: p=%d\n', niter, p);
    end

    % step 1, sampling
    matrix_signal = zeros(p, 2*K+1);
    matrix_h_p = zeros(p, 2*K+1);
    matrix_x = zeros(p, 2*K+1);
    for k = 0:2*K
      x = (0:p-1)'/p + k/S;
      matrix_x(:,k+1) = x;
      matrix_signal(:,k+1) = fct_handle(x);
      if ~isempty(indexset_I)
        matrix_h_p(:,k+1) = matrix_signal(:,k+1) - trig_poly_eval_gaussiannoise(indexset_I,fourier_coeff,0,x);
      else
        matrix_h_p(:,k+1) = matrix_signal(:,k+1);
      end
    end
    nsamples = nsamples + p*(2*K+1);

    if nargout >= 6
      vector_x_all = [vector_x_all; reshape(matrix_x, p * (2*K+1), 1)];
      vector_signal_all = [vector_signal_all; reshape(matrix_signal, p * (2*K+1), 1)];
    end

    % step 2, (2*K+1) many one-dimensional FFTs of length P
    if p == 1
      matrix_h_p_hat = matrix_h_p;
    else
      matrix_h_p_hat = zeros(p, 2*K+1);
      for k = 0:2*K
        matrix_h_p_hat(:,k+1) = p*ifft(matrix_h_p(:,k+1));
      end
    end

    hankel_L = K;
    hankel_N = 2*K+1;
    music_S = S;

    % step 3, for loop over classes/bins P
    for l=0:p-1
      % step 3.1
      max_val = max(abs(matrix_h_p_hat(l+1,:)));
      if max_val/p < epsilon_spatial
        continue
      end
      vector_h = transpose(matrix_h_p_hat(l+1,:));

      % step 3.2
      found_svd = 0;

      % step 3.3, for loop over cut-off parameters epsilon
      for svd_threshold = epsilon_svd_list
        % step 3.3.1, apply MUSIC/ESPRIT algorithm
        switch algorithm_name
          case 'music'
            grid = -[(l:p:S/2-1)'; (l-p:-p:-S/2)'] / S;
            A = exp(2*pi*1i* (0:K-1)' * grid');
            [ sparse_M, sparse_Omega_tilde ] = prony_music_svd( vector_h, hankel_L, hankel_N, music_S, svd_threshold, grid, A );
          otherwise
            [ sparse_M, sparse_Omega_tilde ] = prony_esprit_svd( vector_h, hankel_L, hankel_N, svd_threshold );
        end
        if show_debugmsg
          fprintf('prony_%s_svd l=%2d returned %2d frequencies\n', algorithm_name, l, sparse_M);
        end

        % step 3.3.2
        if sparse_M >= K_SVD_threshold
          if show_debugmsg
            fprintf('frequencies for l=%d not detected or only zero Fourier coefficients', l);
            if niter < niter_max
              fprintf(' => one more iteration');
            end
          fprintf('\n');
          end
          if (p~=1) || (niter_max~=1)
            break;
          end
        end

        % step 3.3.3, compute local frequencies from MUSIC/ESPRIT output
        new_frequencies = round(sparse_Omega_tilde * S);
        new_frequencies(new_frequencies == -S/2) = S/2;

        % step 3.3.4, filter frequencies
        new_frequencies = unique(new_frequencies);
        new_frequencies = new_frequencies(mod(new_frequencies,p) == mod(-l,p));
        if isempty(new_frequencies)
          continue;
        end
        sparse_M = length(new_frequencies);

        % step 3.3.5, compute local Fourier coefficients from small local
        % Vandermonde system
        A_sparse = exp(2*pi*1i* (0:2*K)' * new_frequencies'/S);
        rank_A_sparse = rank(A_sparse);
        if rank_A_sparse ~= sparse_M
          continue;
        end
        if use_l1
          [new_fourier_coeff_mul_p,~,~,info] = spg_bp(A_sparse, vector_h, opts);
          new_fourier_coeff = new_fourier_coeff_mul_p / p;
          if info.stat > 4
            new_fourier_coeff = (A_sparse \ vector_h) / p;
          end
        else
          new_fourier_coeff = (A_sparse \ vector_h) / p;
        end

        % step 3.3.6, check if residual is small enough to exit for loop 3.3
        test_h = A_sparse * new_fourier_coeff;
        ref_h = vector_h/p;
        err_val = max(abs(test_h-ref_h));
        if err_val > 10*epsilon_spatial
          if show_debugmsg
            fprintf('check for l=%d failed, not using detected frequencies', l);
            if niter < niter_max
              fprintf(' => one more iteration');
            end
          fprintf('\n');
          end
          if (p~=1) || (niter_max~=1) || (sparse_M < K_SVD_threshold)
            continue;
          end
        end
        found_svd = 1;
        break;
      end % step 3.3

      % step 3.4
      if ~found_svd
        continue;
      end

      % step 3.5, check if frequency is already known and update in this case
      [~, ia, ib] = intersect(indexset_I,new_frequencies);
      if ~isempty(ia)
        % then sum up corresponding Fourier coefficients
        fourier_coeff(ia) = fourier_coeff(ia) + new_fourier_coeff(ib);
      end

      % step 3.6, append new frequencies to index set of known frequencies
      ib_not = setdiff((1:length(new_frequencies))',ib);
      if ~isempty(ib_not)
        indexset_I = [indexset_I; new_frequencies(ib_not)];
        fourier_coeff = [fourier_coeff; new_fourier_coeff(ib_not)];
      end
    end % step 3

    % step 4, remove Fourier coefficients and frequencies below threshold
    ind = find(abs(fourier_coeff) >= threshold_fc);
    if show_debugmsg
      fprintf('pruning %d small frequencies\n', length(indexset_I) - length(ind));
    end
    indexset_I = indexset_I(ind);
    fourier_coeff = fourier_coeff(ind);

    if isempty(indexset_I)
      continue;
    end

    if show_debugmsg
      fprintf('end   iteration %d: in total %d frequencies detected\n\n', niter, length(indexset_I));
    end

    % step 5, check if global residual is small enough for exiting iteration
    % loop
    vector_x = reshape(matrix_x, p * (2*K+1), 1);
    vector_signal = reshape(matrix_signal, p * (2*K+1), 1);
    check_h = trig_poly_eval_gaussiannoise(indexset_I,fourier_coeff,0,vector_x);
    val = max(abs(check_h-vector_signal));
    if val <= 10*epsilon_spatial
      break;
    end

  end

  % additionally compute Fourier coefficients by solving global Vandermonde
  % system
  if nargout >= 6
    if ~isempty(indexset_I)
      fourier_coeff_full = exp(2*pi*1i* vector_x_all * indexset_I')\vector_signal_all;
    else
      fourier_coeff_full = [];
    end
  end
  niter_used = niter;
end

end


function [ val ] = find_nextprimes( N, d )
% Uses N and computes (d-1) many primes larger than N
%   Input parameters:
%      N - reference number for starting prime search
%      d - number of values to return
%
%   Output parameter:
%      val - row vector of length d, first entry is N and the remaining
%            (d-1) entries are the next prime numbers larger than N
%            in increasing order
  val = zeros(1,d);
  val(1) = simplenextprime(N);
  for i=2:d
    val(i) = simplenextprime(val(i-1));
  end
end

function [y] = simplenextprime(n)
% computes the next prime larger n
% uses simple isprime function of MATLAB
% Author: Lutz Kämmerer

y=n+1;
if mod(y,2)==0,
   y=y+1;
end;

flag=1;
while flag,
   if(isprime(y)),
      return;
   end;
   y=y+2;
end;
end