function [ nfreq_correct, rel_l2_error, nfreq_out, nfreq_extra, niter_used, nsamples, p_vector, max_abs_error, rel_l2_error_full, max_abs_error_full ] = test1d_run_singletest( indexset_in, coeff_in, SNR, algorithm_name, bandwidth_S, initial_P, K, K_SVD_threshold, epsilon_svd_list, epsilon_spatial, threshold_fourier, niter_max )
%TEST1D_RUN_SINGLETEST Runs single test for given 1d signal and parameters.
%   The signal is given as a function handle (black box) to the algorithm.
%
%   Input parameters:
%      indexset_in - frequencies of input signal as column vector
%      coeff_in - corresponding Fourier coefficients as column vector
%      SNR - (linear) signal-to-noise ratio, use value Inf for noiseless case
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      bandwidth_S - parameter for index set (-S/2,...,S/2] of possible
%                    frequencies
%      initial_P - initial FFT length P for first iteration
%      K - size of Hankel matrix used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold - cut-off parameter for number of detected frequencies,
%                        which determines if results of local MUSIC/ESPRIT run
%                        are used, e.g. choose equal to input parameter K
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%      epsilon_spatial - spatial cut-off parameter
%      threshold_fourier - magnitude of smallest possible Fourier coefficient
%      niter_max - maximal number of allowed iterations
%
%   Output parameters:
%      nfreq_correct - number of correctly identified frequencies
%      rel_l2_error - relative l2-error of Fourier coefficients
%      nfreq_out - number of frequencies returned by the algorithm
%      nfreq_extra - nfreq_out minus nfreq_correct
%      niter_used - number of used iterations
%      nsamples - total number of used samples
%      p_vector - vector of length niter_max with FFT lengths
%      max_abs_error - maximal absolute error of Fourier coefficients
%      rel_l2_error_full - updated relative l2-error of Fourier coefficients
%                          computed by solving one large Vandermonde system
%                          which uses all samples and all detected frequencies
%      max_abs_error_full - updated absolute error of Fourier coefficients
%                           computed by solving one large Vandermonde system
%                           which uses all samples and all detected frequencies
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

 
show_debugmsg = 0;
noise_sigma = sqrt(1/SNR)*norm(coeff_in,2);

if length(indexset_in) ~= length(coeff_in)
  error('number of indexset and coefficients differs');
end

if (K==1) && (K_SVD_threshold==1) && (initial_P == bandwidth_S)
  [~, nfft_g] = trig_poly_eval_nfft_gaussiannoise(indexset_in, coeff_in, noise_sigma, 0);
  fct_handle = @(x) trig_poly_eval_nfft_gaussiannoise(indexset_in, coeff_in, noise_sigma, x, nfft_g);
else
  fct_handle = @(varargin) trig_poly_eval_gaussiannoise(indexset_in,coeff_in, noise_sigma, varargin);
end

[ indexset_I, fourier_coeff, niter_used, nsamples, p_vector, fourier_coeff_full ] = prony_sparsefft_iterative_algorithm( fct_handle, algorithm_name, bandwidth_S, initial_P, K, K_SVD_threshold, epsilon_svd_list, epsilon_spatial, threshold_fourier, niter_max, show_debugmsg );

[freq_intersect, ~, ~] = intersect(indexset_in, indexset_I);
nfreq_correct = length(freq_intersect);

[~, ia, ib] = intersect(indexset_in,indexset_I);
long_fhat = coeff_in(:);
long_fhat_out = zeros(size(long_fhat));
long_fhat_out(ia) = fourier_coeff(ib);
long_fhat_out_full = zeros(size(long_fhat));
long_fhat_out_full(ia) = fourier_coeff_full(ib);
freq_neg = setdiff(indexset_I,indexset_in);
freq_missing = setdiff(indexset_in,indexset_I);
if ~isempty(freq_neg)
  [~, ~, ib_ianeg] = intersect(freq_neg,indexset_I);
  long_fhat = [long_fhat; zeros(length(ib_ianeg),1)];
  long_fhat_out = [long_fhat_out; fourier_coeff(ib_ianeg)];
  long_fhat_out_full = [long_fhat_out_full; fourier_coeff_full(ib_ianeg)];
end
max_abs_error = max(abs(long_fhat - long_fhat_out));
rel_l2_error = norm(long_fhat - long_fhat_out,2) / norm(long_fhat,2);
max_abs_error_full = max(abs(long_fhat - long_fhat_out_full));
rel_l2_error_full = norm(long_fhat - long_fhat_out_full,2) / norm(long_fhat,2);
nfreq_out = length(indexset_I);
nfreq_extra = length(indexset_I)-length(freq_intersect);

fprintf('niter: %d, nsamples: %d, nfreq_correct: %d, nfreq_incorrect: %d, %d extra, max abs fc err: %.1e/%.1e, rel. l2-err.: %.1e/%.1e\n', niter_used, nsamples, nfreq_correct, length(indexset_in)-length(freq_intersect), nfreq_extra, max_abs_error, max_abs_error_full, rel_l2_error, rel_l2_error_full);

end

