function [ nfreq_correct_array, all_freq_detected_array, rel_l2_error_array, nfreq_out_array, nfreq_extra_array, niter_used_array, nsamples_array, p_vector ] = test1d_run_singletestset( SNR, bandwidth_S, sparsity_M, type, nruns, algorithm_name, K, K_SVD_threshold, initial_P, niter_max, epsilon_svd_list, epsilon_spatial )
%TEST1_RUN_SINGLETESTSET Runs tests for all one-dimensional signals and one
%   specific algorithm parameter combination. The test results are written into
%   a '.mat' file in the 'results' subdirectory.
%
%   Input parameters:
%      SNR - (linear) signal-to-noise ratio, use value Inf for noiseless case
%      bandwidth_S - parameter for index set (-S/2,...,S/2] of possible
%                    frequencies
%      sparsity_M - signal sparsity M
%      type - type of Fourier coefficients,
%             'unit_circle' for uniform random on the complex unit circle,
%             'box_gr1e-2' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-2
%             'box_gr1e-1' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-1
%      nruns - number of test signals
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      K - size of Hankel matrix used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold - cut-off parameter for number of detected frequencies,
%                        which determines if results of local MUSIC/ESPRIT run
%                        are used, e.g. choose equal to input parameter K
%      initial_P - initial FFT length P for first iteration
%      niter_max - maximal number of allowed iterations
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%      epsilon_spatial - spatial cut-off parameter
%
%   Output parameters:
%      nfreq_correct_array - number of correctly identified frequencies
%                            as a vector of length nruns
%      all_freq_detected_array - vector of length nruns corresponding to each
%                                signal with value 1 if all frequencies were
%                                successfully detected or 0 otherwise
%      rel_l2_error_array - relative l2-errors of Fourier coefficients
%                           for each signal
%      nfreq_out_array - number of frequencies returned by the algorithm
%                        for each signal
%      nfreq_extra_array - nfreq_out minus nfreq_correct for each signal
%      niter_used_array - number of used iterations for each signal
%      nsamples_array - total number of used samples for each signal
%      p_vector - vector of length niter_max with FFT lengths
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

signals_filename = sprintf('signals/signals_S%d_M%d_%s_r%d.mat', bandwidth_S, sparsity_M, type, nruns);
load(signals_filename, 'indexset_cell', 'coeff_cell', 'num_signals', 'S', 'fc_min');
if S ~= bandwidth_S
  error('bandwidth S in file does not match input parameter bandwidth_S');
end

result_filename = sprintf('results/test1d_results_SNR%ddB_%s_S%d_M%d_%s_r%d_p%d_K%d_KSVD%d_niter%d.mat', round(10 * log10(SNR)), algorithm_name, bandwidth_S, sparsity_M, type, nruns, initial_P, K, K_SVD_threshold, niter_max);

if num_signals ~= nruns
  error('nruns input and value in signal file differ');
end

if exist(result_filename, 'file') ~= 0
  %error('result file %s already exists', result_filename);
end

nfreq_correct_array = zeros(nruns, 1);
all_freq_detected_array = zeros(nruns,1);
max_abs_error_array = zeros(nruns,1);
rel_l2_error_array = zeros(nruns,1);
max_abs_error_array_full = zeros(nruns,1);
rel_l2_error_array_full = zeros(nruns,1);
nfreq_out_array = zeros(nruns,1);
nfreq_extra_array = zeros(nruns, 1);
niter_used_array = zeros(nruns, 1);
nsamples_array = zeros(nruns, 1);

M = length(indexset_cell{1});

threshold_fourier = fc_min/10;

for ir=1:nruns
  indexset_in = indexset_cell{ir};
  coeff_in = coeff_cell{ir};
  
  if (length(indexset_in) ~= M) || (length(coeff_in) ~= M)
    error('signal sparsities M differ');
  end

  fprintf('S=%d, M=%d, SNR=%.1e, fc min/max=%.1e/%.1e, algo=%s, p=%d, K=%d, K_svd=%d, ir=%d, epsilon_svd=%.1e-%.1e, eps_spatial=%.1e, thresh_fc=%.1e, niter_max=%d\n', S, M, SNR, min(abs(coeff_in)), max(abs(coeff_in)), algorithm_name, initial_P, K, K_SVD_threshold, ir, max(epsilon_svd_list), min(epsilon_svd_list), epsilon_spatial, threshold_fourier, niter_max);
  [ nfreq_correct, rel_l2_error, nfreq_out, nfreq_extra, niter_used, nsamples, p_vector, max_abs_error, rel_l2_error_full, max_abs_error_full ] = test1d_run_singletest( indexset_in, coeff_in, SNR, algorithm_name, S, initial_P, K, K_SVD_threshold, epsilon_svd_list, epsilon_spatial, threshold_fourier, niter_max );

  nfreq_correct_array(ir) = nfreq_correct;
  max_abs_error_array(ir) = max_abs_error;
  rel_l2_error_array(ir) = rel_l2_error;
  max_abs_error_array_full(ir) = max_abs_error_full;
  rel_l2_error_array_full(ir) = rel_l2_error_full;
  nfreq_out_array(ir) = nfreq_out;
  nfreq_extra_array(ir) = nfreq_extra;
  niter_used_array(ir) = niter_used;
  nsamples_array(ir) = nsamples;

  if nfreq_correct == length(indexset_in)
    all_freq_detected_array(ir) = 1;
  end

  M = sparsity_M;
  save(result_filename, 'nfreq_correct_array', 'all_freq_detected_array', 'nfreq_out_array', 'nfreq_extra_array', 'max_abs_error_array', 'rel_l2_error_array', 'max_abs_error_array_full', 'rel_l2_error_array_full', 'niter_used_array', 'nsamples_array', 'p_vector', 'signals_filename', 'nruns', 'SNR', 'algorithm_name', 'S', 'M', 'initial_P', 'K', 'K_SVD_threshold', 'epsilon_svd_list', 'epsilon_spatial', 'threshold_fourier', 'epsilon_spatial', 'niter_max');
end

end

