function [ out ] = test1d_run_testsets( SNR, bandwidth_S, sparsity_M, type, num_signals, algorithm_name, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list )
%TEST1D_RUN_TESTSETS Runs multiple test sets for all one-dimensional signals and
%   several parameter combination. The test results are written into a '.mat'
%   file in the 'results' subdirectory. Depending on the parameter combinations,
%   this function may take a very very long time.
%
%   Input parameters:
%      SNR - (linear) signal-to-noise ratio, use value Inf for noiseless case
%      bandwidth_S - parameter for index set (-S/2,...,S/2] of possible
%                    frequencies
%      sparsity_M - signal sparsity M
%      type - type of Fourier coefficients,
%             'unit_circle' for uniform random on the complex unit circle,
%             'box_gr1e-2' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-2
%             'box_gr1e-1' for uniform random from box [-1,1]+[-1,1]i with
%             absolute value >= 1e-1
%      num_signals - number of test signals
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      K_array - array of Hankel matrix sizes used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold_array - array of cut-off parameter for number of detected
%                              frequencies, which determines if results of local
%                              MUSIC/ESPRIT run are used,
%                              e.g. choose equal to input parameter K_array
%      P_array - array of initial FFT lengths P for first iteration
%      niter_max - maximal number of allowed iterations
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

out = 0;

epsilon_spatial_MIN = 1e-8;

if isinf(SNR)
  epsilon_spatial = epsilon_spatial_MIN;
else
  noise_sigma = sqrt(1/SNR)*sqrt(sparsity_M);
  epsilon_spatial = 5*noise_sigma;
  if epsilon_spatial < epsilon_spatial_MIN
    epsilon_spatial = epsilon_spatial_MIN;
  end
end

for iK = 1:length(K_array)
  K = K_array(iK);
  K_SVD_threshold = K_SVD_threshold_array(iK);
  for P = P_array
    test1d_run_singletestset( SNR, bandwidth_S, sparsity_M, type, num_signals, algorithm_name, K, K_SVD_threshold, P, niter_max, epsilon_svd_list, epsilon_spatial );
  end
end

end




