function [ nfreq_correct_array, all_freq_detected_array, rel_l2_error_array, nfreq_out_array, nfreq_extra_array, niter_used_array, nsamples_array, p_vector ] = testr1l_symhc_run_singletestset( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, nruns, algorithm_name, K, K_SVD_threshold, initial_P, niter_max, epsilon_svd_list, epsilon_spatial )
%TESTR1L_SYMHC_RUN_SINGLETESTSET Runs tests for all multi-dimensional signals
%   with frequencies within hyperbolic cross index set and one specific
%   algorithm parameter combination. The test results are written into a '.mat'
%   file in the 'results' subdirectory.
%
%   Input parameters:
%      SNR - (linear) signal-to-noise ratio, use value Inf for noiseless case
%      symhc_N - parameter for hyperbolic cross index set
%                   \{k\in Z^d: \prod_{t=1}^d max(1,|k|) <= N \}
%                of possible frequencies
%      d - dimension d
%      lattice_z - generating vector of reconstructing rank-1 lattice for
%                  hyperbolic cross as row vector of length d
%      lattice_S - size of reconstructing rank-1 lattice for hyperbolic cross
%      sparsity_M - signal sparsity M
%      nruns - number of test signals
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      K - size of Hankel matrix used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold - cut-off parameter for number of detected frequencies,
%                        which determines if results of local MUSIC/ESPRIT run
%                        are used, e.g. choose equal to input parameter K
%      initial_P - initial FFT length P for first iteration
%      niter_max - maximal number of allowed iterations
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%      epsilon_spatial - spatial cut-off parameter
%
%   Output parameters:
%      nfreq_correct_array - number of correctly identified frequencies
%                            as a vector of length nruns
%      all_freq_detected_array - vector of length nruns corresponding to each
%                                signal with value 1 if all frequencies were
%                                successfully detected or 0 otherwise
%      rel_l2_error_array - relative l2-errors of Fourier coefficients
%                           for each signal
%      nfreq_out_array - number of frequencies returned by the algorithm
%                        for each signal
%      nfreq_extra_array - nfreq_out minus nfreq_correct for each signal
%      niter_used_array - number of used iterations for each signal
%      nsamples_array - total number of used samples for each signal
%      p_vector - vector of length niter_max with FFT lengths
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

signals_filename = sprintf('signals/signals_symhc_N%d_d%d_M%d_r%d.mat', symhc_N, d, sparsity_M, nruns);
load(signals_filename, 'indexset_cell', 'coeff_cell', 'num_signals', 'd', 'N', 'fc_min');

result_filename = sprintf('results/testr1l_symhc_results_SNR%ddB_%s_S%d_N%d_d%d_M%d_r%d_p%d_K%d_KSVD%d_niter%d.mat', round(10 * log10(SNR)), algorithm_name, lattice_S, N, d, sparsity_M, nruns, initial_P, K, K_SVD_threshold, niter_max);

if N ~= symhc_N
  error('symhc_N input and value in signal file differ');
end

if num_signals ~= nruns
  error('nruns input and value in signal file differ');
end

nfreq_correct_array = zeros(nruns, 1);
all_freq_detected_array = zeros(nruns,1);
max_abs_error_array = zeros(nruns,1);
rel_l2_error_array = zeros(nruns,1);
max_abs_error_array_full = zeros(nruns,1);
rel_l2_error_array_full = zeros(nruns,1);
nfreq_out_array = zeros(nruns,1);
nfreq_extra_array = zeros(nruns, 1);
niter_used_array = zeros(nruns, 1);
nsamples_array = zeros(nruns, 1);

M = size(indexset_cell{1},1);

if sparsity_M ~= M
  error('given sparsity sparsity_M and sparsity in signal file differ');
end

threshold_fourier = fc_min/10;

for ir=1:nruns
  indexset_in = indexset_cell{ir};
  coeff_in = coeff_cell{ir};
  
  if (size(indexset_in,1) ~= M) || (length(coeff_in) ~= M)
    error('signal sparsities M differ');
  end

  fprintf('N=%d, d=%d, M=%d, S=%d, SNR=%.1e, fc min/max=%.1e/%.1e, algo=%s, p=%d, K=%d, K_svd=%d, ir=%d, epsilon_svd=%.1e-%.1e, eps_spatial=%.1e, thresh_fc=%.1e, niter_max=%d\n', N, d, M, lattice_S, SNR, min(abs(coeff_in)), max(abs(coeff_in)), algorithm_name, initial_P, K, K_SVD_threshold, ir, max(epsilon_svd_list), min(epsilon_svd_list), epsilon_spatial, threshold_fourier, niter_max);
  [ nfreq_correct, rel_l2_error, nfreq_out, nfreq_extra, niter_used, nsamples, p_vector, max_abs_error, rel_l2_error_full, max_abs_error_full ] = testr1l_symhc_run_singletest( indexset_in, coeff_in, SNR, algorithm_name, symhc_N, lattice_z, lattice_S, initial_P, K, K_SVD_threshold, epsilon_svd_list, epsilon_spatial, threshold_fourier, niter_max );

  nfreq_correct_array(ir) = nfreq_correct;
  max_abs_error_array(ir) = max_abs_error;
  rel_l2_error_array(ir) = rel_l2_error;
  max_abs_error_array_full(ir) = max_abs_error_full;
  rel_l2_error_array_full(ir) = rel_l2_error_full;
  nfreq_out_array(ir) = nfreq_out;
  nfreq_extra_array(ir) = nfreq_extra;
  niter_used_array(ir) = niter_used;
  nsamples_array(ir) = nsamples;

  if nfreq_correct == size(indexset_in,1)
    all_freq_detected_array(ir) = 1;
  end
  
  save(result_filename, 'nfreq_correct_array', 'all_freq_detected_array', 'nfreq_out_array', 'nfreq_extra_array', 'max_abs_error_array', 'rel_l2_error_array', 'max_abs_error_array_full', 'rel_l2_error_array_full', 'niter_used_array', 'nsamples_array', 'p_vector', 'signals_filename', 'nruns', 'SNR', 'algorithm_name', 'N', 'd', 'lattice_z', 'lattice_S', 'M', 'initial_P', 'K', 'K_SVD_threshold', 'epsilon_svd_list', 'epsilon_spatial', 'threshold_fourier', 'epsilon_spatial', 'niter_max');
end

end

