function [ out ] = testr1l_symhc_run_testsets( SNR, algorithm_name, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, K_array, K_SVD_threshold_array, P_array, niter_max, epsilon_svd_list )
%TESTR1L_SYMHC_RUN_TESTSETS Runs multiple test sets for all multi-dimensional
%   signals with frequencies within hyperbolic cross index set and several
%   parameter combination. The test results are written into a '.mat' file in
%   the 'results' subdirectory. Depending on the parameter combinations,
%   this function may take a very very long time.
%
%   Input parameters:
%      SNR - (linear) signal-to-noise ratio, use value Inf for noiseless case
%      algorithm_name - type of inner algorithm for frequency detection,
%                       either 'music' for MUSIC
%                       or 'esprit' for ESPRIT
%      symhc_N - parameter for hyperbolic cross index set
%                   \{k\in Z^d: \prod_{t=1}^d max(1,|k|) <= N \}
%                of possible frequencies
%      d - dimension d
%      lattice_z - generating vector of reconstructing rank-1 lattice for
%                  hyperbolic cross as row vector of length d
%      lattice_S - size of reconstructing rank-1 lattice for hyperbolic cross
%      sparsity_M - signal sparsity M
%      num_signals - number of test signals
%      K_array - array of Hankel matrix sizes used by MUSIC/ESPRIT,
%          the number of samples in each iteration is FFT length P*(2*K+1)
%      K_SVD_threshold_array - array of cut-off parameter for number of detected
%                              frequencies, which determines if results of local
%                              MUSIC/ESPRIT run are used,
%                              e.g. choose equal to input parameter K_array
%      P_array - array of initial FFT lengths P for first iteration
%      niter_max - maximal number of allowed iterations
%      epsilon_svd_list - list of cut-off parameter epsilon for singular values
%                         of Hankel matrix of local MUSIC/ESPRIT run
%
% Copyright (c) 2016 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

out = 0;

epsilon_spatial_MIN = 1e-8;

if isinf(SNR)
  epsilon_spatial = epsilon_spatial_MIN;
else
  noise_sigma = sqrt(1/SNR)*sqrt(sparsity_M);
  epsilon_spatial = 5*noise_sigma;
  if epsilon_spatial < epsilon_spatial_MIN
    epsilon_spatial = epsilon_spatial_MIN;
  end
end

for iK = 1:length(K_array)
  K = K_array(iK);
  K_SVD_threshold = K_SVD_threshold_array(iK);
  for P = P_array
    testr1l_symhc_run_singletestset( SNR, symhc_N, d, lattice_z, lattice_S, sparsity_M, num_signals, algorithm_name, K, K_SVD_threshold, P, niter_max, epsilon_svd_list, epsilon_spatial );
  end
end

end




