function [ out ] = trig_poly_eval_gaussiannoise( freq, val, sigma, x_or_zMxx )
%TRIG_POLY_EVAL_GAUSSIANNOISE Evaluates multivariate trigonometric polynomial,
%which is given by its frequencies and corresponding Fourier coefficients,
%and perturbed it by additive white Gaussian noise with standard deviation
%sigma.
%   Input parameter:
%      freq - matrix of frequencies with size [num_freq,d]
%      val - array of corresponding Fourier coefficents
%      sigma - standard deviation of Gaussian noise, the mean is zero
%      x_or_zMxx - evaluation nodes
%
%   Input x_or_zMxx is either a matrix with size [num_nodes,d] or a
%   cell with three elements. In the latter case
%      x_or_zMxx{1} - generating vector z of rank-1 lattice of
%                     length t, 1<=t<=d
%      x_or_zMxx{2} - rank-1 lattice size M
%      x_or_zMxx{3} - array (x_{t+1}',...,x_{d}) of length d-t
%                     containing fixed components of the evaluation vector
%      This means that the evaluation nodes are then given by
%      x_j := (j*z(1)/M,...,j*z(t)/M,x_{t+1}',...,x_{d}), j=0,...,M-1.
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

x = [];
if isa(x_or_zMxx,'cell')
  if length(x_or_zMxx) == 3
    z = x_or_zMxx{1};
    M = x_or_zMxx{2};
    xx = x_or_zMxx{3};
    if isempty(xx)
      out = lfft(freq,z,M,val,'notransp');
    else
      out = lfft(freq(:,1:length(z)),z,M,val.*exp(2*pi*1i*freq(:,length(z)+1:size(freq,2))*xx),'notransp');
    end
  else
    x = x_or_zMxx{1};
  end
else
  x = x_or_zMxx;
end

if ~isempty(x)
  out = zeros(size(x,1),1);
  for ifreq = 1:size(freq,1)
    out = out + val(ifreq) * exp(2i*pi*x*freq(ifreq,:)');
  end
end

if sigma > 0
  out = out + complex_gaussian_noise(size(out,1), size(out,2), sigma);
end

end
